












/*----------------------------------------------------------------------- 
 * 
 * File Name: I2DestroyVector.c
 * 
 * Author: Finn, L. S.
 * 
 * Revision: $Id: DestroyVector.m4$
 * 
 *----------------------------------------------------------------------- 
 * 
 * NAME 
 * I2DestroyVector
 * 
 * SYNOPSIS 
 * INT4 I2DestroyVector ( Status *,  INT2Vector **vector );
 * 
 * DESCRIPTION 
 * Returns to system storage allocated by I2CreateVector
 * 
 * DIAGNOSTICS 
 * vector == NULL, *vector == NULL, (*vector)->data == NULL, free failure
 *
 * CALLS
 * LALFree
 * 
 * NOTES
 * 
 *----------------------------------------------------------------------- 
 * 
 * REVISION HISTORY 
 * 
 * $Log: DestroyVector.m4$
 * 
 *-----------------------------------------------------------------------
 */

static volatile const char *DESTROYVECTORC = "$Id: DestroyVector.m4$";

#ifndef _LALSTDLIB_H
#include "LALStdlib.h"
#ifndef _LALSTDLIB_H
#define _LALSTDLIB_H
#endif
#endif

#ifndef _AVFACTORIES_H
#include "AVFactories.h"
#ifndef _AVFACTORIES_H
#define _AVFACTORIES_H
#endif
#endif

typedef INT2 dtype;		/* change for different factory */
typedef INT2Vector vtype;		/* change for different factory */

void I2DestroyVector (Status *status, vtype **vector)
{
  /* 
   * Initialize status
   */

  INITSTATUS (status, DESTROYVECTORC);	
      
  /* 
   * Check vector: is it non-NULL?
   */

  ASSERT (vector != NULL, status, DESTROYVECTOR_EVPTR, DESTROYVECTOR_MSGEVPTR);

  /* 
   * Check vector: does it point to non-NULL?
   */

  ASSERT (*vector != NULL, status, DESTROYVECTOR_EUPTR, DESTROYVECTOR_MSGEUPTR);

  /*
   * Check data in vector: does it point to non-NULL
   */

  ASSERT ((*vector)->data != NULL, status,
          DESTROYVECTOR_EDPTR, DESTROYVECTOR_MSGEDPTR);

  /* Ok, now let's free allocated storage */

  LALFree ( (*vector)->data ); /* free allocated data */
  LALFree ( *vector );	/* free vector struct itself */

  *vector = NULL;		/* make sure we don't point to freed struct */

  RETURN (status);
}
