
#ifndef _GRASP_H
#include "grasp.h"
#ifndef _GRASP_H
#define _GRASP_H
#endif
#endif

#ifndef _LALSTDLIB_H
#include "LALStdlib.h"
#ifndef _LALSTDLIB_H
#define _LALSTDLIB_H
#endif
#endif

#ifndef _FRAMEDATA_H
#include "FrameData.h"
#ifndef _FRAMEDATA_H
#define _FRAMEDATA_H
#endif
#endif

static volatile const char *FRAMEDATAC = "$Id: FrameData.c$";

void
InitializeFrameData (Status *status, FrameDataPar **params)
{
  struct fgetoutput *fgetout;
  struct fgetinput  *fgetinp;

  INITSTATUS (status, FRAMEDATAC);

  /* make sure that argument is not NULL */
  ASSERT (params, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);

  /* make sure that the parameters have not already been assigned */
  ASSERT (*params == NULL, status, FRAMEDATA_ENNUL, FRAMEDATA_MSGENNUL);

  /* assign memory for params */
  *params = (FrameDataPar *) LALMalloc (sizeof (FrameDataPar));

  /* make sure that the allocation was successful */
  ASSERT (*params, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);

  (*params)->fgetoutput = LALMalloc (sizeof (struct fgetoutput));
  (*params)->fgetinput  = LALMalloc (sizeof (struct fgetinput));
  (*params)->first      = 1;
  (*params)->timeLastCalibrated = -1;

  /* make sure allocation was successful */
  ASSERT ((*params)->fgetoutput, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT ((*params)->fgetinput, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);

  fgetout = (struct fgetoutput *) (*params)->fgetoutput;
  fgetinp = (struct fgetinput  *) (*params)->fgetinput;

  fgetinp->nchan      = 1;
  fgetinp->files      = framefiles;
  fgetinp->calibrate  = 1;
  fgetinp->chnames    = (char  **) LALMalloc (fgetinp->nchan*sizeof(char  *));
  fgetinp->locations  = (short **) LALMalloc (fgetinp->nchan*sizeof(short *));
  fgetinp->chnames[0] = "IFO_DMRO";
  fgetinp->inlock     = 1;
  fgetout->npoint     = (int *) LALMalloc (fgetinp->nchan*sizeof(int));
  fgetout->ratios     = (int *) LALMalloc (fgetinp->nchan*sizeof(int));

  /* make sure allocations were successful */
  ASSERT (fgetinp->chnames, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (fgetinp->locations, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (fgetout->npoint, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (fgetout->ratios, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);

  RETURN (status);
}


void
FinalizeFrameData (Status *status, FrameDataPar **params)
{
  struct fgetoutput *fgetout;
  struct fgetinput  *fgetinp;

  INITSTATUS (status, FRAMEDATAC);

  /* make sure that argument is not NULL */
  ASSERT (params, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);

  /* make sure that parameters have been initialized */
  ASSERT (*params, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT ((*params)->fgetoutput, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT ((*params)->fgetinput, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);

  fgetout = (struct fgetoutput *) (*params)->fgetoutput;
  fgetinp = (struct fgetinput  *) (*params)->fgetinput;

  /* LALFree (fgetinp->chnames[0]); */
  LALFree (fgetinp->chnames);
  LALFree (fgetinp->locations);

  LALFree (fgetout->npoint);
  LALFree (fgetout->ratios);
  /* more needs to be freed here */

  LALFree ((*params)->fgetoutput);
  LALFree ((*params)->fgetinput);
  LALFree (*params);
  *params = NULL;

  /* normal exit */
  RETURN (status);
}


void
GetFrameData (
    Status       *status,
    FrameDataOut *output,
    FrameDataInp *input,
    FrameDataPar *params
    )
{
  struct fgetoutput *fgetout;
  struct fgetinput  *fgetinp;

  INT4 code;

  INITSTATUS (status, FRAMEDATAC);

  /* make sure that arguments are not NULL */
  ASSERT (output, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (output->framedata, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (output->framedata->data, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (output->framedata->data->data, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (input, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (params, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (params->fgetoutput, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (params->fgetinput, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);

  fgetout = (struct fgetoutput *) params->fgetoutput;
  fgetinp = (struct fgetinput  *) params->fgetinput;

  if (params->first)
    params->first = 0;

  /* decide whether to seek or not */
  if (input->seek)
  {
    fgetinp->npoint = input->seekPoints;
    fgetinp->seek   = 1;

    /* make sure that data requested is reasonable */
    ASSERT (fgetinp->npoint > 0, status, FRAMEDATA_ESIZE, FRAMEDATA_MSGESIZE);
  }
  else
  {
    fgetinp->npoint = output->framedata->data->length;
    fgetinp->seek   = 0;

    /* make sure that data requested is reasonable */
    ASSERT (fgetinp->npoint > 0, status, FRAMEDATA_ESIZE, FRAMEDATA_MSGESIZE);
  }

  fgetinp->locations[0] = output->framedata->data->data;
  fgetinp->inlock       = 1;

  /* get the data */
  code = fget_ch (fgetout, fgetinp);

  /* make sure that some data was obtained */
  ASSERT (code, status, FRAMEDATA_EDATA, FRAMEDATA_MSGEDATA);

  if (code == 1) /* new locked section */
    output->newLock = 1;
  else /* code == 2: continuing locked section */
    output->newLock = 0;

  /* determine if calibration data has changed since last calibration */
  output->newCal = (params->timeLastCalibrated != fgetout->tcalibrate_gps);

  output->framedata->epoch.gpsSecs  = (INT4)fgetout->tstart_gps;
  output->framedata->epoch.gpsNsecs = (INT4)fmod(1e9*fgetout->tstart_gps, 1e9);

  /* make sure sampling rate is positive */
  ASSERT (fgetout->srate > 0, status, FRAMEDATA_ERATE, FRAMEDATA_MSGERATE);
  output->framedata->deltaT = 1/fgetout->srate;
  output->framedata->f0     = 0;   

  /* probably should assign names, etc. */

  /* normal exit */
  RETURN (status);
}


void
CalibrateFrameData (
    Status                  *status,
    COMPLEX8FrequencySeries *response,
    FrameDataPar            *params
    )
{
  struct fgetoutput *fgetout;
  struct fgetinput  *fgetinp;

  INITSTATUS (status, FRAMEDATAC);

  /* make sure that arguments are not NULL */
  ASSERT (response, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (response->data, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (response->data->data, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (params, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (params->fgetoutput, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);
  ASSERT (params->fgetinput, status, FRAMEDATA_ENULL, FRAMEDATA_MSGENULL);

  /* make sure that the size of the response data is reasonable */
  ASSERT (response->data->length > 1, status, FRAMEDATA_ESIZE, FRAMEDATA_MSGESIZE);

  fgetout = (struct fgetoutput *) params->fgetoutput;
  fgetinp = (struct fgetinput  *) params->fgetinput;

  GRnormalize (fgetout->fri, fgetout->frinum, 2*(response->data->length - 1),
               fgetout->srate, (float *)response->data->data);

  params->timeLastCalibrated = fgetout->tcalibrate_gps;
  response->epoch.gpsSecs    = fgetout->tcalibrate_gps;
  response->epoch.gpsNsecs   = 0;
  response->f0               = 0;
  response->deltaF           = fgetout->srate;

  /* probably should assign names, etc. */

  RETURN(status);
}

